<?php
/*
Plugin Name: Category Featured Image(20script.ir)
Plugin URI: http://20script.ir
Description: Generates different featured image on per category basis

Author: 20script
Version: 1.2

Author URI: http://20script.ir

Copyright 2013  Dr. Tarique Sani  (email : tarique@sanisoft.com)

*/

// Include sidebar
include('includes' . DIRECTORY_SEPARATOR . 'sidebar.php');

// Include custom-post form
include('includes' . DIRECTORY_SEPARATOR . 'custom-post.php');

class CategoryFeaturedImage
{
    /**
     * List of thumbnails extracted from post content.
     *
     * @var array
     */
    private $_extractedThumbnails = array();

    /**
     * Flag to store information if new media manager is enabled or not.
     *
     * @var boolean
     */
    private $_isNewMediaManager = false;

    /**
     * List of default options for plugin.
     *
     * @var array
     */
    private $_options;

    /**
     * List of categories available for post.
     *
     * @var array
     */
    private $_category;

    /**
     * Current tab.
     *
     * @var string
     */
    private $_tab = '';

    /**
     * array of custom post type chosen for apt_pro.
     *
     * @var array
     */
    private $_post_type;

    /**
     * Constructor to initialize plugin options
     *
     * @return void
     */
    public function __construct()
    {
        // WordPress version
        global $wp_version;

        // Set new media manager related flag
        if (0 <= version_compare($wp_version, '3.5'))
        {
            $this->_isNewMediaManager = true;
        }
        else if ($this->__check_dfi_media_context('cfi-dfi'))
        {
            // Handle media tabs to display for default featured image
            add_filter('media_upload_tabs', array($this, 'dfi_media_tabs'));

            // Handle media fields to display for default featured image
            add_filter('attachment_fields_to_edit', array($this, 'dfi_media_fields'), 11, 2);

            // Handle media sent to editor for default featured image
            add_filter('media_send_to_editor', array($this, 'dfi_media_send'), 10, 3);
        }

        // Set options
        $options = get_option('cfi_options');
        if (is_array($options))
        {
            $this->_options = $options;
        }
        if(isset($this->_options['precedence'])){
            $priority = 9;
        } else {
            $priority = 11;
        }

        $this->_post_type = get_option( 'cfi_post_type', array('post') );

        foreach ($this->_post_type as $post_type ) {
            add_action('publish_'.$post_type, array($this, 'cfi_publish_post'), $priority);
        }

        add_action('admin_notices', array($this, 'cfi_check_perms'));
        add_action('admin_menu', array($this, 'cfi_add_admin_menu')); // Add batch process capability
        add_action('admin_enqueue_scripts', array($this, 'cfi_admin_enqueues')); // Plugin hook for adding CSS and JS files required for this plugin
        add_action('wp_ajax_cfi_generatepostthumbnail', array($this, 'cfi_ajax_process_post')); // Hook to implement AJAX request

        // Admin init
        add_action('admin_init', array($this, 'admin_init'));

        //Remove orphan thumbnails
        add_action( 'deleted_post_meta', array($this, 'garbage_collector'));

        // Add meta boxes
        add_action('add_meta_boxes', array($this, 'add_meta_boxes'));

        // Handle auto-skip AJAX request
        add_action('wp_ajax_cfi_handle_auto_skip', array($this, 'cfi_handle_auto_skip'));

        // Load translations
        load_plugin_textdomain('category-featured-image', false, dirname(plugin_basename(__FILE__)) . '/languages/');

        // Add capability to manage plugin
        add_filter('option_page_capability_cfi', array(&$this, 'cfi_option_page_capability'));

        //delete plugin data when unistalled
        register_uninstall_hook(__FILE__, __CLASS__ .'::uninstall_plugin');
    }

    /**
     * Method to delete plugin data when uninstalled
     *
     * @return void
     */
    public static function uninstall_plugin(){
        delete_option( 'cfi_options' );
        delete_option( 'cfi_post_type' );
    }

    /**
     * Method to add capability to manage plugin
     *
     * @param string $capability Capability
     *
     * @return string
     */
    function cfi_option_page_capability($capability)
    {
        return 'manage_cfi';
    }

    /**
     * Register the management page
     *
     * @return void
     */
    public function cfi_add_admin_menu() {
        $capability = (current_user_can('manage_options') ? 'manage_options' : 'manage_cfi');
        add_options_page(__('Category Featured Image Settings', 'category-featured-image'), __('Category Featured Image', 'category-featured-image'), $capability, 'cfi', array($this, 'cfi_interface'));
    }

    /**
     * Create Tabs for setting page
     *
     * @param  string $current tab key
     *
     * @return void
     */
    public function cfi_tabs($current = 'settings') {
        $tabs = array('settings' => 'Settings', 'apply_to' => 'Apply To', 'delete'  => 'Delete Thumbnails');
        if( !array_key_exists($current, $tabs)) {
            $current = 'settings';
        }
        echo "<h2 class='nav-tab-wrapper'>";
        foreach ($tabs as $key => $name ) {
            $class = ($key == $current ) ?'nav-tab-active':'';
            if( $key == 'delete') {
                $class .= ' float-right-nav-tab';
            }
            echo "<a class='nav-tab $class' href='?page=cfi&tab=$key'>$name</a>";
        }
        echo '</h2>';
    }

    /**
     * removes orphan cfi_thumbnail metadata
     *
     * @return void
     */
    public function garbage_collector() {

        global $wpdb;
        $post_id = $wpdb->get_results( "SELECT post_id FROM $wpdb->postmeta WHERE meta_key = '_thumbnail_id'" );

        foreach( $post_id as $post) {
            $post_array[] = $post->post_id;
        }

        if( !empty( $post_array )){
            $post_string = implode(',', $post_array );
        } else {
            $post_string = 0;
        }

            $wpdb->query("DELETE FROM $wpdb->postmeta WHERE meta_key IN ('apt_pro_thumbnail', 'cfi_thumbnail') and post_id NOT IN ($post_string)");

    }

    /**
     * Admin user interface plus post thumbnail generator
     *
     * @return void
     */
    public function cfi_interface() {
        global $wpdb;

        if( isset($_GET['tab'])) {
            $this->_tab = $_GET['tab'];
        } else {
             $this->_tab = 'settings';
        }
        ?>
<div style="margin-right:260px;">
    <div style='float:left; width: 100%'>
        <?php display_advertisement(); ?>
        <div id="message" class="updated fade" style="display:none"></div>
        <div class="wrap">

        <h2><?php printf(__('Category Featured Image - By %s', 'category-featured-image'), '<a href="http://www.sanisoft.com" target="_blank">SANIsoft</a>'); ?></h2>
        <div id="icon-options-general" class="icon32"><br></div>
        <?php
        $this->cfi_tabs( $this->_tab );
        if($this->_tab === 'settings') {
        ?>
        <form action="options.php" method="post">
            <?php settings_fields('cfi_options_group'); ?>
            <?php do_settings_sections('cfi');?>
            <input type="hidden" id="cfi_warn" />
        <?php

        // Button and help text to output
        if ($this->_isNewMediaManager)
        {
            $imageSelector = '<input class="button" id="selector_default_featured_image_#index#" name="selector_button" type="button" value="' . __('Select', 'category-featured-image') . '" />';
        }
        else
        {
            $imageSelectorUrl = get_upload_iframe_src('image', null, 'library');
            $imageSelectorUrl = remove_query_arg(array('TB_iframe'), $imageSelectorUrl);
            $imageSelectorUrl = add_query_arg(array('context' => 'cfi-dfi', 'TB_iframe' => 1, 'width' => 640), $imageSelectorUrl);
            $imageSelector = '<a class="button thickbox" href="' . esc_url($imageSelectorUrl) . '" id="selector_default_featured_image_#index#" title="' . __('Select default featured image', 'category-featured-image') . '">' . __('Select', 'category-featured-image') . '</a>';
        }

        // Output field to set URL of default featured image
         _e('Default Category Featured Image', 'category-featured-image');
         ?>
            </br></br>
            <div id="cfi_form">
                <div id="cfi_form_template">
                <?php $this->category_dropdown( $this->_category ); ?>
                    <input type="text" name="cfi_options[default_featured_image][#index#][url]" id="cfi_form_#index#_url" class="regular-text code" /><?php echo $imageSelector ?><input type="button" name="removeCurrentButton" id ="cfi_form_remove_current" value="<?php _e('Remove', 'category-featured-image'); ?>" class="button-secondary" /></br>
                </div>
                <div id="cfi_form_noforms_template"></div>
                <div id="cfi_form_noforms_controls">
                    <p class="description"><?php  printf(__('Provide a URL for category featured image or choose it from media manager.%s Category with empty  or invalid URL will not be saved.', 'category-featured-image'), '') ?></p>
                    <div>
                        <input type="button" name="addButton" id ="cfi_form_add" value="<?php _e('Add Category', 'category-featured-image'); ?>" class="button-primary" />
                        <input type="button" name="removeButton" id ="cfi_form_remove_last" value="<?php _e('Remove Last Category', 'category-featured-image'); ?>" class="button-secondary" />
                    </div>
                </div>
            </div>

        <?php
        // Show Precedence option if Auto Post Thumbnail Pro is active
        If (is_plugin_active('auto-post-thumbnail-pro/index.php') ) {
            if(!empty($this->_options['precedence'])){
                $checked ='checked ="checked"';
            } else {
                $checked ='';
            }

            echo '<br/><br/><label><input type="checkbox" id="precedence_check" name="cfi_options[precedence]" '.$checked.' /> Take precedence over Auto Post Thumbnail Pro</label>';
        } ?>
            <?php submit_button(); ?>
        </form>
        <script>
        // <![CDATA[

        jQuery(document).ready( function(){

            jQuery.warnUnload({
                message:'',
                urls:[],
                ignore:[".button-primary","#log_mode"]
            });

            jQuery('#cfi_form_add').click(function(){
                jQuery.warnUnload({
                    message:'',
                    urls:[],
                    ignore:[".button-primary"]
                });
            });

            jQuery('#cfi_form_add, #cfi_form_remove_last, #cfi_form_remove_current').click( function() {
                jQuery("#cfi_warn").attr("data-changed","true");
            });
        });

        </script>
        <p class="description">Existing featured images will not be changed. Remember to save your changes before leaving the page</p>
        <?php } elseif($this->_tab === 'delete') { ?>
            <h2><strong>Note:</strong> Use this option very carefully. It cannot be undone by any method except restoring from an older back-up of the database. Also note this option DOES NOT delete images from the media library but just disassociates the featured image link between the post and image.</h2>
                    <hr>
                    <?php
                    //If delete thumbnail button is clicked
                    if( !empty( $_POST['delete-post-thumbnails'])) {
                        $count = -1;
                        //Capability check
                        if ( !current_user_can('manage_options') )
                    wp_die('Cheatin&#8217; uh?');

                        if(isset($_POST['delete_thumbnail'])) {
                            if( $_POST['delete_thumbnail'] == 'all') {
                                $count = $wpdb->get_var("SELECT COUNT(*) FROM $wpdb->postmeta WHERE meta_key = '_thumbnail_id' ");
                                if( $count > 0 ){
                                    $wpdb->query(
                                        "DELETE FROM $wpdb->postmeta WHERE meta_key in ('_thumbnail_id', 'cfi_thumbnail')"
                                    );
                                }

                            } elseif ( $_POST['delete_thumbnail'] == 'plugin') {
                                $count = $wpdb->get_var("SELECT COUNT(*) FROM $wpdb->postmeta WHERE meta_key = 'cfi_thumbnail' ");
                                if( $count > 0 ) {
                                    $post_ids = $wpdb->get_results("SELECT post_id from $wpdb->postmeta WHERE meta_key = 'cfi_thumbnail'", ARRAY_N);

                                    foreach( $post_ids as $post_id) {
                                        $id_array[] = $post_id[0];
                                    }

                                    if( !empty($id_array) ){
                                        $id_string = implode(',', $id_array );

                                        $wpdb->query(
                                                "DELETE FROM $wpdb->postmeta WHERE meta_key in ('_thumbnail_id', 'cfi_thumbnail') and post_id IN ( $id_string )"
                                        );
                                    }
                                }
                            }
                        }
                        if( $count > 0) {
                            $message = "<p><strong>". esc_js( sprintf(__('%d thumbnails deleted.', 'category-featured-image'), $count ) )."</strong></p>";
                        } else if($count < 0) {
                            $message = "<p><strong>". __('Please select an option.', 'category-featured-image')."</strong></p>";
                        } elseif($count == 0) {
                            $message = "<p><strong>". __('No thumbnail to delete.', 'category-featured-image')."</strong></p>";
                        }
                    ?>
                            <script type='text/javascript'>
                                jQuery("#message").html("<?php echo $message; ?>");
                                jQuery("#message").show();
                            </script>
                    <?php
                        } ?>
                    <form action="" method="post">
                        <h3>Delete Thumbnails</h3>
                        <?php

                        $all = $wpdb->get_var("SELECT COUNT(*) FROM $wpdb->postmeta WHERE meta_key = '_thumbnail_id' ");
                        if( $all > 0 ) {
                            $all_msg = sprintf(__( '%d Thumbnails will be deleted', 'category-featured-image' ), $all);
                        } else {
                            $all_msg = __( 'No thumbnail to delete' , 'category-featured-image');
                        }

                        $plugin = $wpdb->get_var("SELECT COUNT(*) FROM $wpdb->postmeta WHERE meta_key = 'cfi_thumbnail' ");
                        if( $plugin > 0 ) {
                            $plugin_msg = sprintf(__( '%d Thumbnails will be deleted', 'category-featured-image' ), $plugin);
                        } else {
                            $plugin_msg = __( 'No thumbnail to delete' , 'category-featured-image');
                        }

                        ?>
                        <label><input type='radio' name='delete_thumbnail' value='all' /><?php printf(__('Delete all Featured Image. (%s)', 'category-featured-image'), $all_msg); ?><br>
                        <p class='description'><?php _e('Choose this option to delete all Featured Image in the blog', 'category-featured-image'); ?></p></label><br>
                        <label><input type='radio' name='delete_thumbnail' value='plugin' /><?php printf(__('Delete Featured Image created by this plugin. (%s)', 'category-featured-image'), $plugin_msg); ?><br>
                        <p class='description'><?php _e('Choose this option to delete Featured Image generated by Category Featured Image, works for thumbnails created using version 1.1 onwards', 'category-featured-image'); ?></p></label><br>
                        <p><input type="submit" class="button-primary" name="delete-post-thumbnails" id="delete-post-thumbnails" value="<?php _e("Delete Thumbnails", "auto-post-thumbnail-pro"); ?>" /></p>

                    </form>
        <?php } elseif($this->_tab === 'apply_to') {
                cfi_apply_to_form();
        } ?>
    </div>
    <?php if( $this->_tab === 'settings') { ?>
    <div class="wrap genpostthumbs">
        <h2><?php _e('Generate Post Thumbnails', 'category-featured-image'); ?></h2>

    <?php
        // If the button was clicked
            if ( !empty($_POST['cfi-generate-post-thumbnails']) ) {
                // Capability check
                if ( !current_user_can('manage_options') )
                    wp_die('Cheatin&#8217; uh?');

                // Form nonce check
                check_admin_referer( 'cfi' );

                foreach($this->_options['default_featured_image'] as $option ){
                    $categories[] = $option['category'];
                }

                $cat_string = implode(',', $categories);

                // Get id's of all the published posts for which post thumbnails does not exist.
                $query = "SELECT * FROM {$wpdb->posts} p where p.post_status = 'publish' AND p.ID NOT IN (
                            SELECT DISTINCT post_id FROM {$wpdb->postmeta} WHERE meta_key IN ('_thumbnail_id', '_cfi_skip_post_thumb')
                          ) AND p.ID IN ( SELECT DISTINCT object_id FROM {$wpdb->term_relationships} WHERE term_taxonomy_id IN ($cat_string)) ORDER BY p.post_date DESC";
                $posts = $wpdb->get_results($query);

                if (empty($posts)) {
                    echo '<p>' . __('Currently there are no published posts available to generate thumbnails.', 'category-featured-image') . '</p>';
                } else {

                    $count = count( $posts );

                    echo '<p>' . sprintf(__('We are generating %d post thumbnails. Please be patient!', 'category-featured-image'), $count) . '</p>';

                    // Generate the list of IDs
                    $ids = array();
                    foreach ( $posts as $post )
                        $ids[] = $post->ID;
                    $ids = implode( ',', $ids );
    ?>
        <noscript><p><em><?php _e('You must enable Javascript in order to proceed!', 'category-featured-image'); ?></em></p></noscript>

        <div id="genpostthumbsbar" style="position:relative;height:25px;">
            <div id="genpostthumbsbar-percent" style="position:absolute;left:50%;top:50%;width:50px;margin-left:-25px;height:25px;margin-top:-9px;font-weight:bold;text-align:center;"></div>
        </div>

        <script type="text/javascript">
        // <![CDATA[
            jQuery(document).ready(function($){
                var i;
                var success = 0;
                var failure = 0;
                var rt_images = [<?php echo $ids; ?>];
                var log_mode = "<?php echo $_POST['log_mode']; ?>";
                var rt_total = rt_images.length;
                var rt_count = 1;
                var rt_percent = 0;
                $("#textarea_log").prop("disabled", "disabled");
                $("#genpostthumbsbar").progressbar();
                $("#genpostthumbsbar-percent").html( "0%" );
                function genPostThumb( id ) {
                    $.post( "admin-ajax.php", { action: "cfi_generatepostthumbnail", id: id }, function(data) {
                        rt_percent = ( rt_count / rt_total ) * 100;
                        $("#genpostthumbsbar").progressbar( "value", rt_percent );
                        $("#genpostthumbsbar-percent").html( Math.round(rt_percent) + "%" );
                        if(data.success ) {
                            if( log_mode == 'full') {
                                $("#textarea_log").append("<?php _e('Successfully generated thumbnail for post ', 'category-featured-image') ?>"+ data.title+'.\n');
                                $("#textarea_log").show();
                            }
                            success+= 1;
                        } else if( !data.success ) {
                            if( log_mode != 'silent' ) {
                                if (data.networkIssue){
                                    var message = '(Category : '+data.category+'). '+"<?php _e('Unable to retrive Image from external source, Connection timeout', 'category-featured-image') ?>";
                                } else {
                                var message = "<?php _e('Image not specified for Category ', 'category-featured-image') ?>"+ ': '+data.category+'.';
                                }
                                $("#textarea_log").append("<?php _e('Failed to generate thumbnail for post ', 'category-featured-image') ?>"+ data.title+' '+message+'\n');
                                $("#textarea_log").show();
                            }

                            failure+= 1;
                        }

                        rt_count = rt_count + 1;

                        if ( rt_images.length ) {
                            genPostThumb( rt_images.shift() );
                        } else {
                            $("#message").html("<p><strong><?php echo esc_js( sprintf(__('All done! Processed %d posts. ', 'category-featured-image'), $count ) ); ?></strong></p>");
                            $("#message").show();
                            $("#warn").attr("data-changed","false");
                            if(success == 0) {
                                $("#message p").append("<strong><?php _e('No thumbnail generated.', 'category-featured-image') ?> </strong>");
                                $("#message").addClass("error");
                                $("#message").removeClass("updated");
                            } else if(failure == 0){
                                $("#message p").append("<strong><?php _e('Thumbnail for all post(s) generated succesfully.', 'category-featured-image') ?> </strong>");
                            } else {
                                $("#message p").append("<strong>"+success+" <?php _e('Thumbnail(s) generated succesfully.', 'category-featured-image') ?> "+failure+" <?php _e('Failed.', 'category-featured-image') ?></strong>");
                                $("#message").addClass("error");
                                $("#message").removeClass("updated");
                            }
                        }

                    });
                }
                jQuery.warnUnload({message:''});
                $("#warn").attr("data-changed","true");
                genPostThumb( rt_images.shift() );
            });
        // ]]>
        </script>
    <?php
            echo '<form><input type="hidden" id="warn" /></form>';
            echo '<textarea id="textarea_log" style="width: 100%; height:200px; display: none;"></textarea>';
                }
            } else {
    ?>

        <p><?php _e('Use this tool to generate Post Thumbnail (Featured Thumbnail) for your Published posts.', 'category-featured-image'); ?></p>
        <p><?php printf(__('If the script stops executing for any reason, just %s the page and it will continue from where it stopped.', 'category-featured-image'), '<strong>' . __('Reload', 'category-featured-image') . '</strong>'); ?></p>

        <form method="post" action="">
    <?php wp_nonce_field('cfi') ?>


        <p><input type="submit" class="button hide-if-no-js" name="cfi-generate-post-thumbnails" id="cfi-generate-post-thumbnails" value="<?php _e('Generate Thumbnails', 'category-featured-image'); ?>" />
        Select log mode:
                <?php
                $mode = array('silent' => __('Silent', 'category-featured-image'), 'error' => __('Errors Only', 'category-featured-image'), 'full' => __('Complete Detail','category-featured-image') );

                foreach ($mode as $key => $name) {
                    $checked = '';
                    if($key == 'silent' ) {
                        $checked = "checked = 'checked'";
                    }
                    echo "<label><input type='radio' id='log_mode' name='log_mode' value='$key' $checked /> $name</label> &nbsp;&nbsp; ";
                }
                ?>
        </p>

        <noscript><p><em><?php _e('You must enable Javascript in order to proceed!', 'category-featured-image'); ?></em></p></noscript>

        </form>
        <div >
        <p><?php printf(__('<strong>Note:</strong> Thumbnails won\'t be generated for posts that already have post thumbnail or setting for skipping thumbnail is on. Also once generated Featured Images can only be deleted by editing individual posts', 'category-featured-image'), '<strong><em>_cfi_skip_post_thumb</em></strong>'); ?></p>
        </div>
        <?php } ?>
        </div>
        <?php } ?>
    </div>
</div>


    <script>
        jQuery("#delete-post-thumbnails").click( function(){
            if( confirm("Do you really want to delete thumbnails?")) {
                return true;
            } else {
                return false;
            }
        });
    </script>
    <?php
    } //End cfi_interface()

    /**
     * Add our JS and CSS files
     *
     * @param string $hook_suffix Hook suffix
     *
     * @return void
     */
    public function cfi_admin_enqueues($hook_suffix) {
        if ( 'settings_page_cfi' != $hook_suffix ) {
            return;
        }

        $taxonomies = get_object_taxonomies( $this->_post_type, $output = 'names' );
        $categories = get_terms( $taxonomies, array('hide_empty' => false) );
        foreach( $categories as $cat ){
            $this->_category[$cat->term_taxonomy_id] = $cat->name;
        }

        // Enqueue media manager scripts
        if ($this->_isNewMediaManager)
        {
            wp_enqueue_media();
            wp_enqueue_script('cfi-options', plugins_url( 'js/admin-options.js', __FILE__));
        }
        else
        {
            wp_enqueue_script('thickbox');
            wp_enqueue_style('thickbox');
        }

        // WordPress 3.1 vs older version compatibility
        if ( wp_script_is( 'jquery-ui-widget', 'registered' ) ) {
            wp_enqueue_script( 'jquery-ui-progressbar', plugins_url( 'js/jquery-ui/jquery.ui.progressbar.min.js', __FILE__ ), array( 'jquery-ui-core', 'jquery-ui-widget' ), '1.7.2' );
        }
        else {
            wp_enqueue_script( 'jquery-ui-progressbar', plugins_url( 'js/jquery-ui/ui.progressbar.js', __FILE__ ), array( 'jquery-ui-core' ), '1.7.2' );
        }

        if( !wp_script_is('underscore', 'registered')) {
            wp_enqueue_script( 'underscore',plugins_url( 'js/underscore-min.js', __FILE__ ));
        }

        wp_enqueue_style( 'jquery-ui-genpostthumbs', plugins_url( 'js/jquery-ui/redmond/jquery-ui-1.7.2.custom.css', __FILE__ ), array(), '1.7.2' );

        if( $_GET['tab'] === 'settings' || !isset($_GET['tab']) ) {

            wp_enqueue_script( 'jQuery-sheepit', plugins_url( 'js/jquery.sheepItPlugin-1.1.1.min.js', __FILE__ ));

            wp_enqueue_script( 'sheepit', plugins_url( 'js/sheepIt.js', __FILE__ ), array('jQuery-sheepit'));

            wp_enqueue_script( 'warnunload', plugins_url( 'js/jquery.warnunload.js', __FILE__ ));

            if( isset($this->_options['default_featured_image']) && !empty($this->_options['default_featured_image'])) {
                $i = 0;
                foreach($this->_options['default_featured_image'] as $option) {
                    if( !in_array($option['category'], array_keys($this->_category ))){
                        unset($this->_options['default_featured_image'][$i]);
                    }
                    $i++;
                }
            }

            wp_localize_script( 'sheepit', 'SheepIt', array(
                        'category' => json_encode($this->_category),
                        'options' => json_encode($this->_options)
                    ));
        }

        wp_enqueue_style( 'cfi_style', plugins_url( 'css/style.css', __FILE__ ));

    } //End cfi_admin_enqueues

    /**
     * Process single post to generate the post thumbnail
     *
     * @return void
     */
    public function cfi_ajax_process_post() {
        if ( !current_user_can( 'manage_options' ) ) {
            die('-1');
        }

        $id = (int) $_POST['id'];

        if ( empty($id) ) {
            die('-1');
        }

        set_time_limit( 60 );

        // Pass on the id to our 'publish' callback function.
        $this->cfi_publish_post($id, true);
        die(-1);
    } //End cfi_ajax_process_post()

    /**
     * Check whether the required directory structure is available so that the plugin can create thumbnails if needed.
     * If not, don't allow plugin activation.
     *
     * @return void
     */
    public function cfi_check_perms() {
        $uploads = wp_upload_dir(current_time('mysql'));

        if ($uploads['error']) {
            echo '<div class="updated"><p>';
            echo $uploads['error'];

            if ( function_exists('deactivate_plugins') ) {
                deactivate_plugins(dirname(plugin_basename(__FILE__)) . '/index.php');
                echo '<br />' . __('This plugin has been automatically deactivated.', 'category-featured-image');
            }

            echo '</p></div>';
        }
    }

    /**
     * Function to generate category thumbnail from default url.
     *
     * @param string $post_id Post ID
     *
     * @return void
     */
    public function cfi_publish_post($post_id, $ajax_request = false )
    {
        global $wpdb;
        $success = false;
        $taxonomies = get_object_taxonomies( $this->_post_type, $output = 'names' );
        $categories = wp_get_object_terms( $post_id, $taxonomies );

        foreach ($categories as $category) {
            $catID[] = $category->term_taxonomy_id;
            $catName[] = $category->name;
        }

        // First check whether Post Thumbnail is already set for this post.
        if (get_post_meta($post_id, '_thumbnail_id', true) || get_post_meta($post_id, '_cfi_skip_post_thumb', true)) {
            return;
        }

        // Initialize a variable to store thumbnail ID
        $thumb_id = false;
        $guid = '';
        $cat_id = '';

        if( !empty($this->_options['default_featured_image']) ) {
            foreach($this->_options['default_featured_image'] as $data ) {
                if( in_array( $data['category'], $catID)){
                    $guid = $data['url'];
                    $cat_id = $data['category'];
                    break;
                }
            }
        }

        if ( !empty($guid))
        {
            $result = $wpdb->get_results("SELECT ID FROM {$wpdb->posts} WHERE guid = '{$guid}' AND post_type = 'attachment'");
            $thumb_id = (isset($result[0]) ? $result[0]->ID : $this->__generate_post_thumb($guid, true));
        }

        // Save generated thumbnail
        if (true === $thumb_id)
        {
            $thumb_id = $this->__save_generated_thumbnail($post_id);
        }

        // If we succeed in generating thumb, let's update post meta
        if ($thumb_id) {
            update_post_meta( $post_id, '_thumbnail_id', $thumb_id );
            update_post_meta( $post_id, 'cfi_thumbnail', $thumb_id );
            $success = true;
        }

        if( $ajax_request ){
            if(!empty($guid)) {
                $networkIssue = true;
            } else {
                $networkIssue = false;
            }

            $title = get_the_title( $post_id );
            if($cat_id != '' ){
                foreach($taxonomies as $taxonomy) {
                    $term = get_term_by( 'id', $cat_id, $taxonomy, OBJECT);
                    if( $term != null) {
                        $cat_name = $term->name;
                    }
                }
                //$cat_name = get_the_category_by_ID( (int)$cat_id );
            } else {
                $cat_name = implode(',', $catName);
            }

            $data = array( 'success'=> $success, 'title' => $title, 'category'=>$cat_name, 'networkIssue' => $networkIssue);
            $response = json_encode($data);

            header( "Content-Type: application/json" );
            echo $response;

            exit;
        }

    }// end cfi_publish_post()

    /**
     * Method used to generate post thumbnail for given image
     *
     * @param string  $imageUrl  Image URL
     * @param boolean $isDefault Flag to decide if given image is default one or not
     *
     * @return boolean
     */
    private function __generate_post_thumb($imageUrl, $isDefault = false)
    {
        // Get the file name
        $filename = substr($imageUrl, (strrpos($imageUrl, '/'))+1);

        // Remove query part from filename
        if (false !== strpos($filename, '?'))
        {
            list($filename) = explode('?', $filename);
        }

        if (!(($uploads = wp_upload_dir(current_time('mysql')) ) && false === $uploads['error'])) {
            return false;
        }

        // Generate unique file name
        $filename = wp_unique_filename( $uploads['path'], $filename );

        // Move the file to the uploads dir
        $new_file = $uploads['path'] . "/$filename";

        $file_data = wp_remote_retrieve_body(wp_remote_get($imageUrl, array('timeout' => 15)));

        if (!$file_data) {
            return false;
        }

        file_put_contents($new_file, $file_data);

        // Set correct file permissions
        $stat = stat( dirname( $new_file ));
        $perms = $stat['mode'] & 0000666;
        @ chmod( $new_file, $perms );

        // Get image's information
        $imageInformation = @getimagesize($new_file);

        // If image is not valid then skip it
        if (false === $imageInformation )
        {
            unlink($new_file);
            return false;
        }

        // Image's extension
        $ext = str_replace(array('image/', 'jpeg'), array('', 'jpg'), $imageInformation['mime']);

        // Add proper extension to attachment
        $filetype = wp_check_filetype($filename);
        if (false === $filetype['ext'] || $ext != $filetype['ext'])
        {
            rename($new_file, $new_file .= '.' . $ext);
            $filename .= '.' . $ext;
        }

        // Compute the URL
        $url = $uploads['url'] . "/$filename";

        // Construct the attachment array
        $attachment = array(
            'post_mime_type' => $imageInformation['mime'],
            'guid' => $url,
            'post_title' => '',
            'post_content' => '',
            'post_status' => 'inherit',
        );

        // Extracted thumbnail details
        $this->_extractedThumbnails['thumb_details'] = compact('attachment', 'new_file');

        // Updating external url with local url
        $i = 0;
        foreach($this->_options['default_featured_image'] as $data) {
            $this->_options['default_featured_image'][$i]['url'] = str_replace($imageUrl, $url, $data['url']);
            $i++;
        }

        update_option( 'cfi_options', $this->_options );
        return true;
    }

    /**
     * Method used to save generated thumbnail for given post ID
     *
     * @param string $post_id Post ID
     *
     * @return integer/null
     */
    private function __save_generated_thumbnail($post_id)
    {
        $thumb_id = wp_insert_attachment($this->_extractedThumbnails['thumb_details']['attachment'], false, $post_id);
        if ( !is_wp_error($thumb_id) ) {
            require_once(ABSPATH . '/wp-admin/includes/image.php');

            // Added fix by misthero as suggested
            wp_update_attachment_metadata( $thumb_id, wp_generate_attachment_metadata( $thumb_id, $this->_extractedThumbnails['thumb_details']['new_file'] ) );
            update_attached_file( $thumb_id, $this->_extractedThumbnails['thumb_details']['new_file'] );

            return $thumb_id;
        }

        return null;
    }

    /**
     * Method used to do things while admin initialization
     *
     * @return void
     */
    public function admin_init()
    {
        register_setting('cfi_options_group', 'cfi_options', array($this, 'sanitize_options'));

        add_settings_section('cfi_options', __('Options', 'category-featured-image'), array($this, 'options_section'), 'cfi');
    }

    /**
     * Method used to output description for section 'options'
     *
     * @return void
     */
    public function options_section()
    {
    }

    /**
     * Creates post category dropdown
     *
     * @param array $cat array containing category id as key and name as value
     *
     * @return void
     */
    public function category_dropdown( $cat ){

        echo '<select id="cfi_form_#index#_category" name="cfi_options[default_featured_image][#index#][category]" >';
        foreach( $cat as $val => $name) {
            echo "<option value=".$val." >".$name."</option>";
        }

        echo '</select>';
    }

    /**
     * Method used to sanitize plugin options
     *
     * @param array $options Options to sanitize
     *
     * @return array
     */
    public function sanitize_options($options)
    {
        //Sanitize 'default featured image' option
        $i = 0;
        foreach($options['default_featured_image'] as $data) {
            $data['url']      = trim((string)$data['url']);
            $data['category'] = (int) $data['category'];

            if (!empty($data['url']) && 0 >= $this->wcs_is_valid_url($data['url'])) {
                unset($data);
                $i++;
            } else {
                if(isset($data) && !empty($data['url'])) {
                     $output[] = array( 'category' => $data['category'], 'url' => $data['url']);
                }

            }
        }

        if($i == 1){
                $cat = __('a category', 'category-featured-image');
        } elseif($i > 1){
                $cat = sprintf(__('%d categories', 'category-featured-image'), $i);
        }

        if( $i > 0){
            add_settings_error('cfi_options', 'default_featured_image', sprintf(__("Invalid URL for %s", 'category-featured-image'), $cat));
        }

        $options['default_featured_image'] = $output;
        return $options;
    }

    /**
     * Method used to check URL's validity
     *
     * @param string  &$url         URL to check its validity
     * @param boolean $check_exists Flag to decide if URL's existence should be checked or not
     *
     * @return boolean
     */
    private function wcs_is_valid_url(&$url, $check_exists=true)
    {
        // result: 1 is returned if good (check for >0 or ==1)
        // result: 0 is returned if syntax is incorrect
        // result: -1 is returned if syntax is correct, but url/file does not exist

        // add http:// (here AND in the referenced $url), if needed
        if (!$url) {return false;}
        if (strpos($url, ':') === false) {$url = 'http://' . $url;}

        // auto-correct backslashes (here AND in the referenced $url)
        $url = str_replace('\\', '/', $url);

        // convert multi-byte international url's by stripping multi-byte chars
        $url_local = urldecode($url) . ' ';
        $len = mb_strlen($url_local);
        if ($len !== strlen($url_local))
        {
            $convmap = array(0x0, 0x2FFFF, 0, 0xFFFF);
            $url_local = mb_decode_numericentity($url_local, $convmap, 'UTF-8');
        }

        $url_local = trim($url_local);

        // now, process pre-encoded MBI's
        $regex = '#&([a-z]{1,2})(?:acute|cedil|circ|grave|lig|orn|ring|slash|th|tilde|uml);#i';
        $url_test = preg_replace($regex, '$1', htmlentities($url_local, ENT_QUOTES, 'UTF-8'));
        if ($url_test != '') {$url_local = $url_test;}

        // test for bracket-enclosed IP address (IPv6) and modify for further testing
        preg_match('#(?<=\[)(.*?)(?=\])#i', $url, $matches);
        if (isset($matches[0]))
        {
            $ip = $matches[0];
            if (!preg_match('/^([0-9a-f\.\/:]+)$/', strtolower($ip))) {return false;}
            if (substr_count($ip, ':') < 2) {return false;}
            $octets = preg_split('/[:\/]/', $ip);
            foreach ($octets as $i) {if (strlen($i) > 4) {return false;}}
            $ip_adj = 'x' . str_replace(':', '_', $ip) . '.com';
            $url_local = str_replace('[' . $ip . ']', $ip_adj, $url_local);
        }

        // test for IP address (IPv4)
        $regex = "^(https?|ftp|news|file)\:\/\/";
        $regex .= "([0-9]{1,3}\.[0-9]{1,3}\.)";
        if (eregi($regex, $url_local))
        {
            $regex = "^(https?|ftps)\:\/\/";
            $regex .= "([0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3})";
            if (!eregi($regex, $url_local)) {return false;}
            $seg = preg_split('/[.\/]/', $url_local);
            if (($seg[2] > 255) || ($seg[3] > 255) || ($seg[4] > 255) || ($seg[5] > 255)) {return false;}
        }

        // patch for wikipedia which can have a 2nd colon in the url
        if (strpos(strtolower($url_local), 'wikipedia'))
        {
            $pos = strpos($url_local, ':');
            $url_left = substr($url_local, 0, $pos + 1);
            $url_right = substr($url_local, $pos + 1);
            $url_right = str_replace(':', '_', $url_right);
            $url_local = $url_left . $url_right;
        }

        // construct the REGEX for standard processing
        // scheme
        $regex = "^(https?|ftp|news|file)\:\/\/";
        // user and password (optional)
        $regex .= "([a-z0-9+!*(),;?&=\$_.-]+(\:[a-z0-9+!*(),;?&=\$_.-]+)?@)?";
        // hostname or IP address
        $regex .= "([a-z0-9+\$_-]+\.)*[a-z0-9+\$_-]{2,4}";
        // port (optional)
        $regex .= "(\:[0-9]{2,5})?";
        // dir/file path (optional)
        $regex .= "(\/([a-z0-9+\$_-]\.?)+)*\/?";
        // query (optional)
        $regex .= "(\?[a-z+&\$_.-][a-z0-9;:@/&%=+\$_.-]*)?";
        // anchor (optional)
        $regex .= "(#[a-z_.-][a-z0-9+\$_.-]*)?\$";

        // test it
        $is_valid = eregi($regex, $url_local) > 0;

        // final check for a TLD suffix
        if ($is_valid)
        {
            $url_test = str_replace('-', '_', $url_local);
            $regex = '#^(.*?//)*([\w\.\d]*)(:(\d+))*(/*)(.*)$#';
            preg_match($regex, $url_test, $matches);
            $is_valid = preg_match('#^(.+?)\.+[0-9a-z]{2,4}$#i', $matches[2]) > 0;
        }

        // check if the url/file exists
        if (($check_exists) && ($is_valid))
        {
            $status = array();
            $url_test = str_replace(' ', '%20', $url);
            $request = wp_remote_get($url_test);
            if (!is_wp_error($request) && $request['response']['code'] == 200) {$is_valid = true;}
            else {$is_valid = -1;}
        }

        // exit
        return $is_valid;
    }

    /**
     * Method used to add meta boxes for 'post'
     *
     * @param string $screen Screen type
     *
     * @return void
     */
    public function add_meta_boxes($screen)
    {
        // Add meta box for 'selected post types' add/edit
        if ( in_array($screen, $this->_post_type) )
        {
            add_meta_box('cfi', __('Category Featured Image', 'category-featured-image'), array($this, 'add_meta_box'), $screen, 'side', 'core');
        }
    }

    /**
     * Method used to output meta box
     *
     * @param object $post Post details
     *
     * @return void
     */
    public function add_meta_box($post)
    {
        // Get needed meta for post
        $skipPostThumb = get_post_meta($post->ID, '_cfi_skip_post_thumb', true);
?>
        <label for="cfi_handle_auto_skip">
            <input type="checkbox" id="cfi_handle_auto_skip" <?php checked(empty($skipPostThumb), false); ?> />
            <?php _e('Skip automatic thumbnail generation', 'category-featured-image'); ?>
        </label>
        <script type="text/javascript">
        // <![CDATA[
            function cfi_attachAutoSkipHandlingEvent(checkboxReference)
            {
                jQuery.get('admin-ajax.php', { action: 'cfi_handle_auto_skip', id: <?php echo $post->ID; ?>, cfi_skip: (checkboxReference.checked ? 1 : 0) });
            }

            jQuery(document).ready(function($)
            {
                if ('undefined' == typeof(jQuery.on))
                {
                    jQuery('#cfi').delegate('#cfi_handle_auto_skip', 'click', function()
                    {
                        cfi_attachAutoSkipHandlingEvent(this);
                    });
                }
                else
                {
                    jQuery('#cfi').on('click', '#cfi_handle_auto_skip', function()
                    {
                        cfi_attachAutoSkipHandlingEvent(this);
                    });
                }
            });
        // ]]>
        </script>
<?php
    }

    /**
     * Method used to handle auto-skip thumbnail generation
     *
     * @return void
     */
    public function cfi_handle_auto_skip()
    {
        // Check if logged in user can manage options or not
        if (!current_user_can('manage_options'))
        {
            return;
        }

        // Check for empty post ID
        $id = (int)$_GET['id'];
        if (empty($id)) {
            return;
        }

        // Skip or use automatic thumbnail generation
        if ((bool)$_GET['cfi_skip'])
        {
            update_post_meta($id, '_cfi_skip_post_thumb', 1);
        }
        else
        {
            delete_post_meta($id, '_cfi_skip_post_thumb');
        }
    }

    /**
     * Method used to build media tabs for default featured image
     *
     * @param array $tabs Media tabs
     *
     * @return array
     */
    public function dfi_media_tabs($tabs)
    {
        // We need only 'library' and 'type' media tabs
        foreach ($tabs as $type => $title)
        {
            if (!in_array($type, array('library', 'type')))
            {
                unset($tabs[$type]);
            }
        }

        // Return built tabs
        return $tabs;
    }

    /**
     * Method used to manage fields to display for media
     *
     * @param array  $fields Fields to display
     * @param object $post   Attachment post object
     *
     * @return array
     */
    public function dfi_media_fields($fields, $post)
    {
        // Remove all existing fields
        $fields = array();

        // Build new fields
        if (false !== strpos($post->post_mime_type, 'image'))
        {
            $send = "<input type='submit' class='button' name='send[$post->ID]' value='" . esc_attr__('Use as Default Featured Image', 'category-featured-image') . "' />";
            $fields['buttons'] = array('tr' => "\t\t<tr class='submit'><td></td><td class='savesend'>$send</td></tr>\n");
            $fields['context'] = array('input' => 'hidden', 'value' => 'cfi-dfi');
            $fields['image-size'] = array('input' => 'hidden', 'value' => 'full');
            $fields['post_title'] = array('input' => 'hidden', 'value' => $post->post_title);
            $fields['url'] = array('input' => 'hidden', 'value' => $post->guid);
        }
        else
        {
            $fields['buttons'] = array('tr' => '<tr><td colspan="2"></td></tr>');
        }

        // Return built fields
        return $fields;
    }

    /**
     * Method used to set URL for default featured image
     *
     * @param string  $html       HTML to display
     * @param integer $sendId     Attachment ID
     * @param array   $attachment Attachment details
     *
     * @return void
     */
    public function dfi_media_send($html, $sendId, $attachment)
    {
?>
<script type="text/javascript">
/* <![CDATA[ */
var win = window.dialogArguments || opener || parent || top;
target = win.jQuery('.target');
target.val("<?php echo $attachment['url']; ?>");
target.attr("data-changed", "true");
win.tb_remove();
/* ]]> */
</script>
<?php
        exit();
    }

    /**
     * Method used to check media context for default featured image
     *
     * @param string $context Context for media
     *
     * @return boolean
     */
    private function __check_dfi_media_context($context)
    {
        // Add needed filter for given context
        if (isset($_REQUEST['context']) && $_REQUEST['context'] == $context)
        {
            add_filter('media_upload_form_url', array($this, 'add_my_context_to_url'), 10, 2);
            return true;
        }

        return false;
    }

    /**
     * Method used to add media context to URL for default featured image
     *
     * @param string $url  URL to add context for media
     * @param string $type Type of media
     *
     * @return string
     */
    public function add_my_context_to_url($url, $type)
    {
        // Add 'context' query argument, if needed
        if ('image' == $type && isset($_REQUEST['context']))
        {
            $url = add_query_arg('context', $_REQUEST['context'], $url);
        }

        return $url;
    }
}

new CategoryFeaturedImage;
